<?php
// +-----------------------------------------------------------------------+
// | Piwigo - a PHP based picture gallery                                  |
// +-----------------------------------------------------------------------+
// | Copyright(C) 2008      Piwigo Team                  http://piwigo.org |
// | Copyright(C) 2003-2008 PhpWebGallery Team    http://phpwebgallery.net |
// | Copyright(C) 2002-2003 Pierrick LE GALL   http://le-gall.net/pierrick |
// +-----------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or modify  |
// | it under the terms of the GNU General Public License as published by  |
// | the Free Software Foundation                                          |
// |                                                                       |
// | This program is distributed in the hope that it will be useful, but   |
// | WITHOUT ANY WARRANTY; without even the implied warranty of            |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      |
// | General Public License for more details.                              |
// |                                                                       |
// | You should have received a copy of the GNU General Public License     |
// | along with this program; if not, write to the Free Software           |
// | Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, |
// | USA.                                                                  |
// +-----------------------------------------------------------------------+
if (!defined('PHPWG_ROOT_PATH'))
   die("Hacking attepmt!");
include_once( PHPWG_ROOT_PATH.'include/common.inc.php' );

check_status(ACCESS_GUEST);

$username = !empty($_POST['username'])?$_POST['username']:$user['username'];
$mail_address = !empty($_POST['mail_address'])?$_POST['mail_address']:@$user['mail_address'];
$name = !empty($_POST['name'])?$_POST['name']:'';
$author = !empty($_POST['author'])?$_POST['author']:'';
$date_creation = !empty($_POST['date_creation'])?$_POST['date_creation']:'';
$comment = !empty($_POST['comment'])?$_POST['comment']:'';

//------------------------------------------------------------------- functions
// The validate_upload function checks if the image of the given path is valid.
// A picture is valid when :
//     - width, height and filesize are not higher than the maximum
//       filesize authorized by the administrator
//     - the type of the picture is among jpg, gif and png
// The function returns an array containing :
//     - $result['type'] contains the type of the image ('jpg', 'gif' or 'png')
//     - $result['error'] contains an array with the different errors
//       found with the picture
function validate_upload( $filePtr, $my_max_file_size,
                          $image_max_width, $image_max_height )
{
  global $conf, $lang, $page, $mail_address;

  $result = array();
  $result['errors'] = array();
  $extension = get_extension( $filePtr['name'] );
  $temp_name = $filePtr['tmp_name'];
  if (!in_array($extension, $conf['picture_ext']))
  {
    array_push( $result['errors'], l10n('upload_advise_filetype') );
    return $result;
  }
  if ( !isset( $filePtr ) )
  {
    // do we even have a file?
    array_push( $result['errors'], "You did not upload anything!" );
  }
  else if ( $filePtr['size'] > $my_max_file_size * 1024 )
  {
    array_push( $result['errors'],
                l10n('upload_advise_filesize').$my_max_file_size.' KB' );
  }
  else
  {
      // check if we are allowed to upload this file_type
      // upload de la photo sous un nom temporaire
      $size = getimagesize( $temp_name );
      if ( isset( $image_max_width )
            and $image_max_width != ""
            and $size[0] > $image_max_width )
      {
         array_push( $result['errors'],
                     l10n('upload_advise_width').$image_max_width.' px' );
      }
      if ( isset( $image_max_height )
            and $image_max_height != ""
            and $size[1] > $image_max_height )
      {
         array_push( $result['errors'],
                     l10n('upload_advise_height').$image_max_height.' px' );
      }
      // $size[2] == 1 means GIF
      // $size[2] == 2 means JPG
      // $size[2] == 3 means PNG
      switch ( $size[2] )
      {
      case 1 : $result['type'] = 'gif'; break;
      case 2 : $result['type'] = 'jpg'; break;
      case 3 : $result['type'] = 'png'; break;
      default :
         array_push( $result['errors'], l10n('upload_advise_filetype') );
      }
  }
  if ( sizeof( $result['errors'] ) > 0 )
  {
    // destruction de l'image avec le nom temporaire
    @unlink( $temp_name );
  }
  else
  {
    @chmod( $temp_name, 0644);
  }

  //------------------------------------------------------------ log informations
  pwg_log();

  return $result;
}

function sessDataUnpack($sessData)
{
   if(!is_string($sessData))
      return false;

   $data = preg_split('#([\w\d]+?)\|#',$sessData,-1,PREG_SPLIT_DELIM_CAPTURE | PREG_SPLIT_NO_EMPTY);

   $flip = true;
   $sessData = array();
   foreach($data as $bit)
   {
      if($flip)
         $key = $bit;
      else
         $sessData[$key] = unserialize($bit);
         $flip = !$flip;
   }

   return $sessData;
}

//-------------------------------------------------- access authorization check
function validate_autorization($sessionId,$categoryId) {
   global $user;

   $result = array();
   $result['errors'] = array();

   $data = pwg_session_read($sessionId);

   if (trim($data) == NULL)
      array_push($result['errors'],'Invalid session ID');

   if (!is_numeric($categoryId))
   {
      array_push($result['errors'],'You did not select a category!');
      return $result;
   }

   if (in_array($categoryId, explode(',',$user['forbidden_categories'])))
      array_push($result['errors'], 'Access denied for selected category');

   return $result;
}

function do_upload($categoryId,$sessionId,$filePtr) {

   global $conf;

   $result = array();
   $result['errors'] = array();

   $category = get_cat_info( $categoryId );
   $category['cat_dir'] = get_complete_dir($categoryId);

   if (url_is_remote($category['cat_dir']) or !$category['uploadable'])
   {
      array_push($result['errors'], 'upload not allowed');
      return $result;
   }

   $filePath = $category['cat_dir'].'/'.$filePtr['name'];
   $destination = realpath(PHPWG_ROOT_PATH.$category['cat_dir']).'/'.$filePtr['name'];

   if (!move_uploaded_file($filePtr['tmp_name'],$destination))
      array_push($result['errors'], 'unable to upload file');

   $result['uploaded_file'] = $destination;
   $result['web_file_path'] = $filePath;

   //TODO: submit file in to images and image_category tables

   return $result;
}

function do_submitToDatabase($categoryId, $filePtr, $filePath) {

   $query = 'INSERT INTO ' . IMAGES_TABLE .
            ' ( file , date_available , tn_ext, path, storage_category_id ) ' .
            ' VALUES ' .
            ' ( "' . $filePtr['name'] . '", "' . date('Y-m-d H:i:s') . '" , "jpg" , "' . $filePath . '" , "' . $categoryId . '");';

   pwg_query($query);
   $imageId = mysql_insert_id();

   $query = 'INSERT INTO ' . IMAGE_CATEGORY_TABLE .
            ' ( image_id , category_id ) ' .
            ' VALUES ' .
            ' ( "' . $imageId . '" , "' . $categoryId . '");';

   pwg_query($query);

   $result = array();
   $result['errors']  = array();
   $result['imageId'] = $imageId;

   return $result;
}

function do_submitToWaiting($categoryId, $sessionId, $filePtr) {

   global $conf;

   $result = array();
   $result['errors'] = array();

   $fileName = $filePtr['name'];

   $category = get_cat_info( $categoryId );
   $category['cat_dir'] = get_complete_dir($categoryId);

   // Get info about user
   $data = sessDataUnpack(pwg_session_read($sessionId));
   $userData = getuserdata($data['pwg_uid'],false);

   //-------------------------------------------------------------- picture upload
   // verfying fields
   $path = $category['cat_dir'].$fileName;
   if ( @is_file( $path ) )
   {
      array_push( $result['errors'], l10n('upload_file_exists') );
   }

   // test de la prsence des champs obligatoires
   if ( empty($fileName))
   {
      array_push( $result['errors'], l10n('upload_filenotfound') );
   }

   // creation of the "infos" field :
   // <infos author="Pierrick LE GALL" comment="my comment"
   //        date_creation="2004-08-14" name="" />
   $xml_infos = '<infos';
   $xml_infos.= encodeAttribute('author',$userData['username']);
   $xml_infos.= encodeAttribute('comment','');
   $xml_infos.= encodeAttribute('date_creation','');
   $xml_infos.= encodeAttribute('name','');
   $xml_infos.= ' />';

   if ( !preg_match( '/^[a-zA-Z0-9-_.]+$/', $fileName['name'] ) )
   {
      array_push( $result['errors'], l10n('update_wrong_dirname') );
   }

   if ( sizeof( $result['errors'] ) == 0 )
   {
      $query = 'insert into '.WAITING_TABLE;
      $query.= ' (storage_category_id,file,username,mail_address,date,infos)';
      $query.= ' values ';
      $query.= '('.$categoryId.",'".$fileName."'";
      $query.= ",'".htmlspecialchars($userData['username'])."'";
      $query.= ",'".$userData['email']."',".time().",'".$xml_infos."')";
      $query.= ';';
      pwg_query( $query );
      $result['waiting_id'] = mysql_insert_id();

      if ($conf['email_admin_on_picture_uploaded'])
      {
         include_once(PHPWG_ROOT_PATH.'include/functions_mail.inc.php');

         $waiting_url = get_absolute_root_url().'admin.php?page=upload';

         $keyargs_content = array
         (
         get_l10n_args('Category: %s', get_cat_display_name($category['upper_names'], null, false)),
         get_l10n_args('Picture name: %s', $fileName),
         get_l10n_args('User: %s', $userData['username']),
         get_l10n_args('Email: %s', $userData['email']),
         get_l10n_args('Picture name: %s', $fileName),
         get_l10n_args('Author: %s', $userData['username']),
         get_l10n_args('', ''),
         get_l10n_args('Waiting page: %s', $waiting_url)
         );

         pwg_mail_notification_admins
         (
         get_l10n_args('Picture uploaded by %s', $userData['username']),
         $keyargs_content
         );
      }
   }

   return $result;
}

function do_ApproveWaiting($waitingId) {
   $query = 'UPDATE ' . WAITING_TABLE .
            'SET validated = "true"' .
            'WHERE id = ' . $waitingId . ';';

   pwg_query($query);
}

function do_thumbnail($waiting_id,$path,$newWidth,$newHeight,$tn_ext)
{
   //------------------------------------------------------------ create thumbnail

   $result = array();
   $result['errors'] = array();

   RatioResizeImg($path, $newWidth, $newHeight, $tn_ext,$result['errors']);

   if (sizeof($result['errors']) == 0)
   {

      // set the thumbnail
      $query = 'select file';
      $query.= ' from '.WAITING_TABLE;
      $query.= ' where id = '.$waiting_id;
      $query.= ';';
      $query_result= pwg_query( $query );
      $row = mysql_fetch_array( $query_result );
      $file = substr ( $row['file'], 0, strrpos ( $row['file'], ".") );
      $extension = $tn_ext;

      if ( sizeof( $result['errors'] ) == 0 )
      {
         $query = 'update '.WAITING_TABLE;
         $query.= " set tn_ext = '".$extension."'";
         $query.= ' where id = '.$waiting_id;
         $query.= ';';
         pwg_query( $query );
      }
   }

   return($result);
}

//------------------------------------------------------------------- functions
// RatioResizeImg creates a new picture (a thumbnail since it is supposed to
// be smaller than original picture !) in the sub directory named
// "thumbnail".
function RatioResizeImg($path, $newWidth, $newHeight, $tn_ext, &$errors)
{
  global $conf, $lang, $page;

  if (!function_exists('gd_info'))
  {
     array_push($errors,'GD Not installed');
    return;
  }

  $filename = basename($path);
  $dirname = dirname($path);
  
  // extension of the picture filename
  $extension = get_extension($filename);

  if (in_array($extension, array('jpg', 'JPG', 'jpeg', 'JPEG')))
  {
    $srcImage = @imagecreatefromjpeg($path);
  }
  else if ($extension == 'png' or $extension == 'PNG')
  {
    $srcImage = @imagecreatefrompng($path);
  }
  else
  {
    unset($extension);
  }

  if ( isset( $srcImage ) )
  {
    // width/height
    $srcWidth    = imagesx( $srcImage ); 
    $srcHeight   = imagesy( $srcImage ); 
    $ratioWidth  = $srcWidth/$newWidth;
    $ratioHeight = $srcHeight/$newHeight;

    // maximal size exceeded ?
    if ( ( $ratioWidth > 1 ) or ( $ratioHeight > 1 ) )
    {
      if ( $ratioWidth < $ratioHeight)
      { 
        $destWidth = $srcWidth/$ratioHeight;
        $destHeight = $newHeight; 
      }
      else
      { 
        $destWidth = $newWidth; 
        $destHeight = $srcHeight/$ratioWidth;
      }
    }
    else
    {
      $destWidth = $srcWidth;
      $destHeight = $srcHeight;
    }
    // according to the GD version installed on the server
    if ( gdVersion() >= 2 )
    {
      // GD 2.0 or more recent -> good results (but slower)
      $destImage = imagecreatetruecolor( $destWidth, $destHeight); 
      imagecopyresampled( $destImage, $srcImage, 0, 0, 0, 0,
                          $destWidth,$destHeight,$srcWidth,$srcHeight );
    }
    else
    {
      // GD prior to version  2 -> pretty bad results :-/ (but fast)
      $destImage = imagecreate( $destWidth, $destHeight);
      imagecopyresized( $destImage, $srcImage, 0, 0, 0, 0,
                        $destWidth,$destHeight,$srcWidth,$srcHeight );
    }

    if (($tndir = mkget_thumbnail_dir($dirname, $errrors)) == false)
    {
      return false;
    }

    $dest_file = $tndir.'/'.$conf['prefix_thumbnail'];
    $dest_file.= get_filename_wo_extension($filename);
    $dest_file.= '.'.$tn_ext;
    
    // creation and backup of final picture
    if (!is_writable($tndir))
    {
      array_push($errors, '['.$tndir.'] : '.l10n('no_write_access'));
      return false;
    }
    imagejpeg($destImage, $dest_file);
    // freeing memory ressources
    imagedestroy( $srcImage );
    imagedestroy( $destImage );
    
    list($tn_width, $tn_height) = getimagesize($dest_file);
    $tn_size = floor(filesize($dest_file) / 1024).' KB';
    
    $info = array( 'path'      => $path,
                   'tn_file'   => $dest_file,
                   'tn_width'  => $tn_width,
                   'tn_height' => $tn_height,
                   'tn_size'   => $tn_size );
    return $info;
  }
  // error
  else
  {
    array_push($errors, l10n('tn_no_support'));
    if ( isset( $extenstion ) )
    {
      array_push($errors,l10n('tn_format').' '.$extension);
    }
    else
    {
      array_push($errors,l10n('tn_thisformat'));
    }
    return;
  }
}

// From http://cn2.php.net/manual/en/function.gd-info.php#52481
function gdVersion($user_ver = 0)
{
    if (! extension_loaded('gd')) { return; }
    static $gd_ver = 0;
    // Just accept the specified setting if it's 1.
    if ($user_ver == 1) { $gd_ver = 1; return 1; }
    // Use the static variable if function was called previously.
    if ($user_ver !=2 && $gd_ver > 0 ) { return $gd_ver; }
    // Use the gd_info() function if possible.
    if (function_exists('gd_info')) {
        $ver_info = gd_info();
        preg_match('/\d/', $ver_info['GD Version'], $match);
        $gd_ver = $match[0];
        return $match[0];
    }
    // If phpinfo() is disabled use a specified / fail-safe choice...
    if (preg_match('/phpinfo/', ini_get('disable_functions'))) {
        if ($user_ver == 2) {
            $gd_ver = 2;
            return 2;
        } else {
            $gd_ver = 1;
            return 1;
        }
    }
    // ...otherwise use phpinfo().
    ob_start();
    phpinfo(8);
    $info = ob_get_contents();
    ob_end_clean();
    $info = stristr($info, 'gd version');
    preg_match('/\d/', $info, $match);
    $gd_ver = $match[0];
    return $match[0];
} // End gdVersion()
?>
